#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"
#include <iostream>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/format.hpp>

using namespace cv;
using namespace std;
namespace bpt = boost::posix_time;


Mat img;
int threshval = 112;

const char* keys =
{
    "{1| |stuff.jpg|image for converting to a grayscale}"
    "{2| |1000|loops}"
    "{3| |8|connectivity}"
};

int main( int argc, const char** argv )
{
    CommandLineParser parser(argc, argv, keys);
    string inputImage = parser.get<string>("1");
    img = imread(inputImage.c_str(), 0);
    int iterations = parser.get<int>("2");
    int connectivity= parser.get<int>("3");

    if(img.empty())
    {
        cout << "Could not read input image file: " << inputImage << endl;
        return -1;
    }

    Mat bw = threshval < 128 ? (img < threshval) : (img > threshval);

    bpt::ptime start_old = bpt::microsec_clock::universal_time();
    if(connectivity == 8){
        vector<vector<Point> > contours;
        vector<Vec4i> hierarchy;
        for(int t = 0; t < iterations; ++t){
            contours.clear();
            hierarchy.clear();

            findContours( bw, contours, hierarchy, CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE );

            Mat dst = Mat::zeros(img.size(), CV_8UC3);

            if( !contours.empty() && !hierarchy.empty() )
            {
                // iterate through all the top-level contours,
                for(int idx = 0 ; idx >= 0; idx = hierarchy[idx][0] )
                {
                    Scalar color((idx>>16) & 255, (idx>>8) & 255, (idx>>0) & 255);
                    drawContours( dst, contours, idx, color, CV_FILLED, 8, hierarchy );
                }
            }
        }
    }
    bpt::ptime end_old = bpt::microsec_clock::universal_time();

    bpt::ptime start_new = bpt::microsec_clock::universal_time();
    {
        Mat labelImage(img.size(), CV_32S);
        for(int t = 0; t < iterations; ++t){
            uint64_t nLabels = connectedComponents(labelImage, bw, connectivity);
            (void) nLabels;
        }
    }
    bpt::ptime end_new = bpt::microsec_clock::universal_time();

    bpt::ptime start_new_stats = bpt::microsec_clock::universal_time();
    {
        Mat labelImage(img.size(), CV_32S);
        std::vector<ConnectedComponentStats> statsv;
        for(int t = 0; t < iterations; ++t){
            uint64_t nLabels = connectedComponents(labelImage, bw, statsv, connectivity);
            (void) nLabels;
        }
    }
    bpt::ptime end_new_stats = bpt::microsec_clock::universal_time();

    //cout<<boost::format("input: %s connectivity: %d iterations: %d  old method: %7.4fs new method: %7.4fs new method w/ stats: %7.4fs")%iterations
    cout<<boost::format("% 20s %01d % 10d %7.4f %7.4f %7.4f")
    //cout<<boost::format("Iterations: %d old method: %7.4fs new method: %7.4fs")%iterations
        %inputImage
        %connectivity
        %iterations
        %((end_old - start_old).total_milliseconds()/1000.0)
        %((end_new - start_new).total_milliseconds()/1000.0)
        %((end_new_stats - start_new_stats).total_milliseconds()/1000.0)
        <<endl
    ;

    return 0;
}
